;/*******************************/;
;/* Demo Timer MCS51 For ET-EXP4*/;
;/* Files Name  : LAB10EX3.ASM  */;
;/* CPU Control : MCS-51 Family */;
;/* Assembler   : SXA51.EXE     */;
;/* -Timer Run  : Interupt Mode */;
;/* (Adjust %Duty Cycle of PWM) */;
;/*******************************/;
;
;/* Port For Generate PWM
PORT_PWM    EQU     P1.0                    ; Port.0 = PWM Output

            ORG     20H                     ; Internal RAM of CPU
FLAG_BUF:   DS      1                       ; Flag Status
PWMP1P:     DS      1                       ; % PWM Buffer (1-256 Cycle)
PWMDC1:     DS      1                       ; % Duty Cycle of PWM
OFFLO:      DS      2                       ; High Pulse Time Offset
OFFHI:      DS      2                       ; Low Pulse Time Offset
            ;
PWM_STAS    EQU     FLAG_BUF.0              ; Set = 1,Clr = 0

            ORG     0000H                   ; Reset Vector of CPU
            LJMP    MAIN
            ORG     000BH                   ; Timer0 Vector Interupt
            LJMP    TIMER_INT

MAIN:       MOV     SP,#128-32              ; Stack = 32 Byte Internal RAM
            CLR     PWM_STAS
            MOV     TMOD,#00000001B         ; Timer0 Mode1 (16Bit)
            SETB    ET0                     ; Enable Timer0 Interupt
            SETB    TR0                     ; Strat Timer0
            SETB    EA                      ; Enable Global Interupt
            ;
            ;/* "PWMP1P" : Set % PWM Cycle (1-256) */;
            MOV     PWMP1P,#92              ; 1% PWM = 10milliSec(100Hz)
            ;
            ;/* Generate PWM on P1.0 */;
STRT_PWM:   MOV     PWMDC1,#2               ; Start PWM Duty Cycle = 2%
            ;
INC_PWM:    LCALL   UPDATE_PWM              ; Update PWM
            LCALL   DELAY                   ; Delay Output PWM Signal
            ;
            INC     PWMDC1
            MOV     A,PWMDC1
            CJNE    A,#98,INC_PWM           ; Loop Gen. PWM Until 98%
            SJMP    STRT_PWM                ; PWM = 98% Restart at 2%

;/***************************/;
;/* Math SUB 16Bit 2-Number */;
;/* Input  : R2,R3          */;
;/*        : R4,R5          */;
;/* Output : R2,R3          */;
;/* (R2,R3 - R4,R5 = R2,R3) */;
;/***************************/;
;
SUB_16:     CLR     C
            MOV     A,R3
            SUBB    A,R5                    ; Low Byte
            MOV     R3,A
            MOV     A,R2
            SUBB    A,R4                    ; High Byte + Carry
            MOV     R2,A
            RET

;/*******************/;
;/* Update PWM Time */;
;/*******************/;
;
UPDATE_PWM: MOV     A,PWMDC1                ; High Pulse Time
            MOV     B,PWMP1P                ; % PWM Cycle
            MUL     AB
            MOV     OFFHI+0,B               ;/* Save High Pulse Cycle Time */;
            MOV     OFFHI+1,A
            ;
            MOV     A,PWMP1P                ; % PWM Cycle
            MOV     B,#100                  ; 100 % PWM Cycle (Total Pulse)
            MUL     AB
            MOV     R2,B                    ; 100% Pulse
            MOV     R3,A
            MOV     R4,OFFHI                ; High Pulse
            MOV     R5,OFFHI
            LCALL   SUB_16                  ; 100% Pulse - High Pulse = Low Pulse
            MOV     OFFLO+0,R2              ;/* Save Low Pulse Cycle Time */;
            MOV     OFFLO+1,R3
            RET

;/*****************************/;
;/* Timer 0 Interrupt Service */;
;/* Auto Reload on Interrupt  */;
;/*****************************/;
;
TIMER_INT:  JB      PWM_STAS,NEXT_LOW
            MOV     A,OFFHI+0               ; 1'Complement
            CPL     A
            MOV     DPH,A
            MOV     A,OFFHI+1
            CPL     A
            MOV     DPL,A
            INC     DPTR                    ; 2'Complement
            MOV     TH0,DPH                 ; Reload Time High
            MOV     TL0,DPL
            SETB    PORT_PWM
            SJMP    END_TIME
            ;
NEXT_LOW:   MOV     A,OFFLO+0               ; 1'Complement
            CPL     A
            MOV     DPH,A
            MOV     A,OFFLO+1
            CPL     A
            MOV     DPL,A
            INC     DPTR                    ; 2'Complement
            MOV     TH0,DPH                 ; Reload Time Low
            MOV     TL0,DPL
            CLR     PORT_PWM
END_TIME:   CPL     PWM_STAS                ; Next Status
            RETI

;/**************/;
;/* Delay Time */;
;/**************/;
;
DELAY:      MOV     R4,#100
DLY1:       MOV     R5,#250
            DJNZ    R5,$
            DJNZ    R4,DLY1
            RET

            END


            


