;/*********************************/;
;/* Demo Dot LCD 16X2 For ET-EXP4 */;
;/* Files Name  : LAB2-EX3.ASM    */;
;/* CPU Control : MCS51 Family    */;
;/* Assembler   : SXA51.EXE       */;
;/*  (Shift Message Display LCD)  */;
;/*********************************/;
;
;/* Port For Control LCD
PORT_LCD    EQU     P1                  ; Port Data LCD
RS_LCD      EQU     PORT_LCD.0          ; RS Select
E_LCD       EQU     PORT_LCD.2          ; Enable LCD

            ORG     0000H               ; Start at Monitor
START:      MOV     SP,#128-32          ; Stack = 32 Byte
            CLR     E_LCD               ; Disable LCD
            MOV     R2,#0FFH            ; Power-on Delay
PWR_DLY:    MOV     R3,#0FFH
            DJNZ    R3,$
            DJNZ    R2,PWR_DLY
            LCALL   INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
LOOP:       MOV     DPTR,#TAB_MSG1
            MOV     B,#16               ; 16 Character Write LCD
            MOV     A,#06H              ; Entry mode set
            LCALL   WR_INS              ; I/D=1 Increment,S=0 cursor shift
            MOV     A,#00H              ; First Cursor Line1
            LCALL   GOTO_LCD
DSP_LFT:    CLR     A
            MOVC    A,@A+DPTR           ; Write Left to Right
            LCALL   WR_LCD
            LCALL   MOVE_DLY
            INC     DPTR
            DJNZ    B,DSP_LFT
            ;
            MOV     DPTR,#TAB_MSG2+15
            MOV     B,#16               ; 16 Character Write LCD
            MOV     A,#04H              ; Entry mode set
            LCALL   WR_INS              ; I/D=0 Decrease,S=0 cursor shift
            MOV     A,#4FH              ; Last Cursor Line2
            LCALL   GOTO_LCD
DSP_RGT:    CLR     A
            MOVC    A,@A+DPTR           ; Display Right to Left
            LCALL   WR_LCD
            LCALL   MOVE_DLY
            LCALL   DEC_DPTR
            DJNZ    B,DSP_RGT
            ;
            MOV     B,#16               ; 16 Character Shift Left
SHF_LFT:    MOV     A,#18H              ; Insert Left
            LCALL   WR_INS
            LCALL   MOVE_DLY
            DJNZ    B,SHF_LFT
            LCALL   DELAY               ; Delay 2 Second
            ;
            MOV     B,#16               ; 16 Character Shift Right
SHF_RGT:    MOV     A,#1CH              ; Insert Right
            LCALL   WR_INS
            LCALL   MOVE_DLY
            DJNZ    B,SHF_RGT
            LCALL   DELAY               ; Delay 2 Second
            ;
            MOV     B,#16               ; 16 Character Shift Right
SHF_RGT2:   MOV     A,#1CH              ; Insert Right
            LCALL   WR_INS
            LCALL   MOVE_DLY
            DJNZ    B,SHF_RGT2
            LCALL   DELAY               ; Delay 2 Second
            ;
            LCALL   CLR_LCD             ; Clear Display LCD
            LJMP    LOOP

TAB_MSG1:   DB      'DEMO LCD ET-EXP4'
TAB_MSG2:   DB      ' BY ETT CO.,LTD '

;/**************/;
;/* Delay Time */;
;/**************/;
;
DELAY:      MOV     R2,#10              ; Delay Count
DLY1:       MOV     R3,#0FFH
DLY2:       MOV     R4,#0FFH
            DJNZ    R4,$
            DJNZ    R3,DLY2
            DJNZ    R2,DLY1
            RET

;/**************/;
;/* Delay Move */;
;/**************/;
;
MOVE_DLY:   MOV     R2,#0FFH
MOVE_DL1:   MOV     R3,#0FFH
            DJNZ    R3,$
            DJNZ    R2,MOVE_DL1
            RET

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PX.0  */;
;/* -> E    = PX.2  */;
;/* -> D4-7 = PX4-7 */;
;/*******************/;
;
INIT_LCD:   MOV     A,#33H              ; DL = 1 ,DL =1
            LCALL   WR_INS
            MOV     A,#32H              ; DL = 1 ,DL =0
            LCALL   WR_INS
            MOV     A,#28H              ; Function set
            LCALL   WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            MOV     A,#0CH              ; Display on/off Control
            LCALL   WR_INS              ; Entry display,cursor off,not blink
            MOV     A,#06H              ; Entry mode set
            LCALL   WR_INS              ; I/D=1 Increment,S=0 cursor shift
CLR_LCD:    MOV     A,#01H              ; Clear Display
            LCALL   WR_INS              ; Clear Display,Set DD RAM addr=0
            RET

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PX.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD:     PUSH    B
            MOV     B,#50               ; Busy Delay LCD
            DJNZ    B,$
            POP     B
            SETB    E_LCD               ; Enable LCD (E=PX.2=1)
            NOP
            CLR     E_LCD               ; Disable LCD (E=PX.2=0)
            RET

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PUSH    ACC                 ; Save Data Low Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; High Byte & RS=1 Write
            SETB    RS_LCD              ; Set RS For Write Data Select
            LCALL   EN_LCD
            POP     ACC                 ; Restore Data Low Byte
            SWAP    A                   ; Shift Low Byte to High Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; Low Byte & RS=1 Write
            SETB    RS_LCD              ; Set RS For Write Data Select
            LCALL   EN_LCD
            RET

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS:     PUSH    ACC                 ; Save Data Low Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; High Byte & RS=0 Write
            LCALL   EN_LCD
            ;
            POP     ACC                 ; Restore Data Low Byte
            SWAP    A                   ; Shift Low Byte to High Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; Low Byte & RS=0 Write
            LCALL   EN_LCD
            MOV     R2,#2               ; Busy Ins Delay
INS_DLY:    MOV     R3,#250
            DJNZ    R3,$
            DJNZ    R2,INS_DLY
            RET
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD:   SETB    ACC.7
            LCALL   WR_INS
            RET

;/************************************/;
;/*  Print Data data to Serial Port  */;
;/*  to display buffer.              */;
;/*  Usage    : LCALL PRINT_LCD      */;
;/*           : DB   'xxxx',00       */;
;/*  Register : ACC                  */;
;/*  Note     : last byte must be 00 */;
;/************************************/;
;
PRINT_LCD:  POP     DPH
            POP     DPL
PRINT1:     CLR     A
            MOVC    A,@A+DPTR
            CJNE    A,#00H,PRINT2
            SJMP    PRINT3
PRINT2:     LCALL   WR_LCD
            INC     DPTR
            SJMP    PRINT1
PRINT3:     PUSH    DPL
            PUSH    DPH
            RET

;/*****************/;
;/* DPTR = DPTR-1 */;
;/* In,Out = DPTR */;
;/* A (ZERO FLAG) */;
;/*****************/;
;
DEC_DPTR:   XCH     A,DPL
            JNZ     DEC_DPL
            DEC     DPH
DEC_DPL:    DEC     A
            XCH     A,DPL
            RET

            END



  

