;/********************************/;
;/* Demo Scankey 4x3 For ET-EXP4 */;
;/* Files Name  : LAB4-EX2.ASM   */;
;/* CPU Control : MCS51 Family   */;
;/* Assembler   : SXA51.EXE      */;
;/*  (Scankey & Display on LCD)  */;
;/********************************/;
;
;/* Port Scankey Matrix 4x3
PORT_KEY    EQU     P0                  ; Port Scan Keyboard
;
;/* Port For Control LCD
PORT_LCD    EQU     P1                  ; Port Data LCD
RS_LCD      EQU     PORT_LCD.0          ; RS Select
E_LCD       EQU     PORT_LCD.2          ; Enable LCD

            ORG     20H
FLAG_BUF:   DS      1                   ; Flag Status
ROW:        DS      1                   ; Row Buffer
COLUM:      DS      1                   ; Colum Buffer
STACK:      DS      32
;                                       ; Stack 32 Byte
KEY_PRES    EQU     FLAG_BUF.0
KEY_STAS    EQU     FLAG_BUF.1

            ORG     0000H               ; Start at Monitor
START:      MOV     SP,#STACK           ; Stack = 32 Byte
            CLR     E_LCD               ; Disable LCD
            MOV     R2,#0FFH            ; Power-on Delay
PWR_DLY:    MOV     R3,#0FFH
            DJNZ    R3,$
            DJNZ    R2,PWR_DLY
            LCALL   INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
            MOV     FLAG_BUF,#0
WAIT_KEY:   LCALL   SCANKEY             ; Scankey 4X3
            CJNE    A,#0FFH,DSP_KEY1
            SJMP    WAIT_KEY            ; Loop Until Keypress
            ;
DSP_KEY1:   CJNE    A,#0AH,DSP_KEY2     ; Convert to ASCII
DSP_KEY2:   JNC     DSP_KEY3
            ADD     A,#30H              ; Convert 0-9 to ASCII
            LCALL   WR_LCD
            SJMP    WAIT_KEY
            ;
DSP_KEY3:   CLR     CY
            SUBB    A,#09H              ; Convert A-F to ASCII
            ADD     A,#40H
            LCALL   WR_LCD
            SJMP    WAIT_KEY

;/********************************/;
;/*  Scan Keyboard Matrix  (4x3) */;
;/*  Output    : ACC (code 0..F) */;
;/*            : FFH (Not press) */;
;/*  Register  : ACC,B,R2,DPTR   */;
;/*  Sub. Call : KEY_DLY         */;
;/*            : KEY_NEW         */;
;/********************************/;
;
SCANKEY:    MOV     ROW,#3              ; Counter for Scan 4x3
            MOV     COLUM,#11111110B    ; Data for Scan Colum
SCAN1:      MOV     PORT_KEY,COLUM      ; Out Colum (P0-P2)
            MOV     A,PORT_KEY          ; In Row (P4-P7)
            ANL     A,#0F0H
            CJNE    A,#0F0H,PRESS       ; Check row only (PC4-PC7)
            MOV     A,COLUM             ; Scan Next Colum
            RL      A
            MOV     COLUM,A
            DJNZ    ROW,SCAN1
            MOV     A,#0FFH             ; Not Keypress
            CLR     KEY_PRES            ; not key Press
            SJMP    END_SCAN
            ;
PRESS:      LCALL   KEY_DLY             ; Debounce key
            MOV     PORT_KEY,COLUM      ; Out Colum (P0-P2)
            MOV     A,PORT_KEY          ; In Row (P4-P7)
            ANL     A,#0F0H
            CJNE    A,#0F0H,KEY_CODE    ; Row (PC4-PC7)
            SJMP    SCANKEY             ; No keybounce or noise
KEY_CODE:   LCALL   KEY_NEW
            JNB     KEY_STAS,SCAN1      ; if old key don't care
            MOV     B,A                 ; Row code (High)
            MOV     A,COLUM
            ANL     A,#0FH              ; Colum (Low Byte)
            ORL     A,B                 ; Mix Row & Colum
            MOV     B,A                 ; Row & Colum code
            ;
            MOV     DPTR,#KEY_TAB
            MOV     R2,#0FFH
            MOV     ROW,#12             ; 12 Key Maximum
CODE1:      INC     R2
            MOV     A,R2
            MOVC    A,@A+DPTR
            CJNE    A,B,CODE2
            MOV     A,R2                ; Number code 0..F
            SJMP    END_SCAN
            ;
CODE2:      DJNZ    ROW,CODE1
            MOV     A,#0FFH             ; Not Match Code
            CLR     KEY_PRES            ; not key Press
END_SCAN:   RET

;/**********************************/;
;/*     Check Old & New key        */;
;/*  KEY_STAS = 0 is old press Key */;
;/*  KEY_STAS = 1 is 1st press Key */;
;/**********************************/;
;
KEY_NEW:    CLR     KEY_STAS            ; Next time = old key
            JB      KEY_PRES,END_NEW    ; Jump if old key press
            SETB    KEY_PRES            ; Next time = old press
            SETB    KEY_STAS            ; Next time = old key
END_NEW:    RET

;/****************/;
;/*   Delay For  */;
;/* Debounce key */;
;/****************/;
;
KEY_DLY:    MOV     R6,#01FH
KEY_DL1:    MOV     R7,#0FFH
            DJNZ    R7,$
            DJNZ    R6,KEY_DL1
            RET

;/*********************/;
;/* Table of Key Code */;
;/*********************/;
;
KEY_TAB:    DB      07DH,0EEH,0EDH,0EBH     ; 0 1 2 3
            DB      0DEH,0DDH,0DBH,0BEH     ; 4 5 6 7
            DB      0BDH,0BBH,07EH,07BH     ; 8 9 * #

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PX.0  */;
;/* -> E    = PX.2  */;
;/* -> D4-7 = PX4-7 */;
;/*******************/;
;
INIT_LCD:   MOV     A,#33H              ; DL = 1 ,DL =1
            LCALL   WR_INS
            MOV     A,#32H              ; DL = 1 ,DL =0
            LCALL   WR_INS
            MOV     A,#28H              ; Function set
            LCALL   WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            MOV     A,#0FH              ; Display on/off Control
            LCALL   WR_INS              ; Entry display,cursor on,blink
            MOV     A,#06H              ; Entry mode set
            LCALL   WR_INS              ; I/D=1 Increment,S=0 cursor shift
            MOV     A,#01H              ; Clear Display
            LCALL   WR_INS              ; Clear Display,Set DD RAM addr=0
            RET

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PX.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD:     PUSH    B
            MOV     B,#50               ; Busy Delay LCD
            DJNZ    B,$
            POP     B
            SETB    E_LCD               ; Enable LCD (E=PX.2=1)
            NOP
            CLR     E_LCD               ; Disable LCD (E=PX.2=0)
            RET

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PUSH    ACC                 ; Save Data Low Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; High Byte & RS=1 Write
            SETB    RS_LCD              ; Set RS For Write Data Select
            LCALL   EN_LCD
            POP     ACC                 ; Restore Data Low Byte
            SWAP    A                   ; Shift Low Byte to High Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; Low Byte & RS=1 Write
            SETB    RS_LCD              ; Set RS For Write Data Select
            LCALL   EN_LCD
            RET

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS:     PUSH    ACC                 ; Save Data Low Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; High Byte & RS=0 Write
            LCALL   EN_LCD
            ;
            POP     ACC                 ; Restore Data Low Byte
            SWAP    A                   ; Shift Low Byte to High Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; Low Byte & RS=0 Write
            LCALL   EN_LCD
            MOV     R2,#2               ; Busy Ins Delay
INS_DLY:    MOV     R3,#250
            DJNZ    R3,$
            DJNZ    R2,INS_DLY
            RET
            
            END



            












