;/*********************************/;
;/* Demo Dot LCD 16X2 For ET-EXP4 */;
;/* Files Name  : LAB2-EX2.ASM    */;
;/* CPU Control : 68HC811E2       */;
;/* Assembler   : AS11NEW.EXE     */;
;/*  (Change 2-Display Message)   */;
;/*********************************/;
;
;/* Port For Control LCD (Port-B)
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)

            ORG     $F800               ; Start at Monitor 68HC811E2
START       LDS     #$FF                ; Stack = Last Internal RAM
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
LOOP        LDAA    #$00                ; 1st Location Line1
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'DEMO LCD ET-EXP4'  ; Message on Line-1
            FCB     $00
            LDAA    #$40                ; 1st Location Line2
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     ' BY ETT CO.,LTD '  ; Message on Line-2
            FCB     $00
            JSR     DELAY               ; Delay 5 Second
            ;
            LDAA    #$00                ; 1st Location Line1
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     '16 Char x 2 Line'  ; Message on Line-1
            FCB     $00
            LDAA    #$40                ; 1st Location Line2
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'Interface 4-Bits'  ; Message on Line-2
            FCB     $00
            JSR     DELAY               ; Delay 5 Second
            ;
            BRA     LOOP

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PB0   */;
;/* -> E    = PB2   */;
;/* -> D4-7 = PB4-7 */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0C                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor off,not blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
            LDAA    #$01                ; Clear Display
            JSR     WR_INS              ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      PSHB
            LDAB    #20                 ; 100uS counter
BUSY        DECB
            BNE     BUSY
            PULB
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0632              ; 5 mSec counter
INS_DLY     DEX
            BNE     INS_DLY
            PULX
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

;/********************************/;
;/* Print ASCII data to LCD Port */;
;/* Usage    : JSR  PRINT_LCD    */;
;/*          : FCC  'xxxxxxxx'   */;
;/*          : FCB  $00          */;
;/* Note     : last byte = $00   */;
;/********************************/;
;
PRINT_LCD   PULX                        ; Pop Stack
            PSHA
            PSHB
PRINT1      LDAA    0,X                 ; Get Data For Print
            CMPA    #$00                ; Check Last Byte
            BNE     PRINT2              ; Jump if not Last Byte
            BRA     PRINT3
PRINT2      JSR     WR_LCD              ; Write Data to LCD
            INX
            BRA     PRINT1
PRINT3      INX
            PULB
            PULA
            PSHX                        ; Push Stack
            RTS

;/**********************/;
;/* Delay Time 2Second */;
;/**********************/;
;
DELAY       PSHB                        ; Save Register
            PSHX
            LDAB    #200                ; 10mS x 200 = 2Second
DLY1        LDX     #$0D06              ; 10mS Count
DLY2        DEX
            BNE     DLY2
            DECB
            BNE     DLY1
            PULX                        ; Restore Register
            PULB
            RTS

            ORG     $FFFE               ; Reset Vector
            FDB     START

            END



