;/*********************************/;
;/* Demo Dot LCD 16X2 For ET-EXP4 */;
;/* Files Name  : LAB2-EX3.ASM    */;
;/* CPU Control : 68HC811E2       */;
;/* Assembler   : AS11NEW.EXE     */;
;/*  (Shift Message Display LCD)  */;
;/*********************************/;
;
;/* Port For Control LCD (Port-B)
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)

            ORG     $F800               ; Start at Monitor 68HC811E2
START       LDS     #$FF                ; Stack = Last Internal RAM
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
LOOP        LDX     #TAB_MSG1
            LDAB    #16                 ; 16 Character Write LCD
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
            LDAA    #$00                ; First Cursor Line1
            JSR     GOTO_LCD
DSP_LFT     LDAA    0,X                 ; Write Left to Right
            JSR     WR_LCD
            JSR     MOVE_DLY
            INX
            DECB
            BNE     DSP_LFT
            ;
            LDX     #TAB_MSG2+15
            LDAB    #16                 ; 16 Character Write LCD
            LDAA    #$04                ; Entry mode set
            JSR     WR_INS              ; I/D=0 Decrease,S=0 cursor shift
            LDAA    #$4F                ; Last Cursor Line2
            JSR     GOTO_LCD
DSP_RGT     LDAA    0,X                 ; Display Right to Left
            JSR     WR_LCD
            JSR     MOVE_DLY
            DEX
            DECB
            BNE     DSP_RGT
            ;
            LDAB    #16                 ; 16 Character Shift Left
SHF_LFT     LDAA    #$18                ; Insert Left
            JSR     WR_INS
            JSR     MOVE_DLY
            DECB
            BNE     SHF_LFT
            JSR     DELAY               ; Delay 2 Second
            ;
            LDAB    #16                 ; 16 Character Shift Right
SHF_RGT     LDAA    #$1C                ; Insert Right
            JSR     WR_INS
            JSR     MOVE_DLY
            DECB
            BNE     SHF_RGT
            JSR     DELAY               ; Delay 2 Second
            ;
            LDAB    #16                 ; 16 Character Shift Right
SHF_RGT2    LDAA    #$1C                ; Insert Right
            JSR     WR_INS
            JSR     MOVE_DLY
            DECB
            BNE     SHF_RGT2
            JSR     DELAY               ; Delay 2 Second
            ;
            JSR     CLR_LCD             ; Clear Display LCD
            BRA     LOOP

TAB_MSG1    FCC     'DEMO LCD ET-EXP4'
TAB_MSG2    FCC     ' BY ETT CO.,LTD '

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PB0   */;
;/* -> E    = PB2   */;
;/* -> D4-7 = PB4-7 */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0C                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor off,not blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
CLR_LCD     LDAA    #$01                ; Clear Display
            JSR     WR_INS              ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      PSHB
            LDAB    #20                 ; 100uS counter
BUSY        DECB
            BNE     BUSY
            PULB
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0D06              ; 5 mSec counter
INS_DLY     DEX
            BNE     INS_DLY
            PULX
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

;/**********************/;
;/* Delay Time 2Second */;
;/**********************/;
;
DELAY       PSHB                        ; Save Register
            PSHX
            LDAB    #200                ; 10mS x 200 = 2Second
DLY1        LDX     #$0D06              ; 10mS Count
DLY2        DEX
            BNE     DLY2
            DECB
            BNE     DLY1
            PULX                        ; Restore Register
            PULB
            RTS

;/*******************/;
;/* Delay Time 50mS */;
;/*******************/;
;
MOVE_DLY    PSHB                        ; Save Register
            PSHX
            LDAB    #10                 ; 10mS x 5 = 50mS
MOVE_DL1    LDX     #$0D06              ; 10mS Count
MOVE_DL2    DEX
            BNE     MOVE_DL2
            DECB
            BNE     MOVE_DL1
            PULX                        ; Restore Register
            PULB
            RTS

            ORG     $FFFE               ; Reset Vector
            FDB     START

            END

