;/********************************/;
;/* Dip-Switch Input For ET-EXP4 */;
;/* Files Name  : LAB4-EX1.ASM   */;
;/* CPU Control : 68HC811E2      */;
;/* Assembler   : AS11NEW.EXE    */;
;/* (Show Latch Input SW on LCD) */;
;/********************************/;
;
;/* Port Read Input Dip-SW4 (Port-C)
PORT_INP    EQU     $1003               ; Port-C
CTRL_INP    EQU     $1007               ; DDRC 0=In,1=Out
;
;/* Port For Control LCD (Port-B)
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)

            ORG     $00
INPUT_BUF   RMB     1                   ; Input Read Buffer

            ORG     $F800               ; Start Monitor of 68H1811E2
START       LDS     #$FF                ; Stack on Last Internal RAM
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
            LDAA    #%00001111          ; PCH=In,PCL=Out
            STAA    CTRL_INP
            LDAA    #$F0
            STAA    INPUT_BUF
            ;
            LDAA    #$00                ; 1st Location Line1
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'Dip-SW4Bit Input'  ; Message on Line-1
            FCB     $00
            LDAA    #$40                ; 1st Location Line2
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'Input SW4 :     '  ; Message on Line-2
            FCB     $00
            BRA     SHOW_INP            ; Show Logic Input
            ;
READ_INP    LDAA    PORT_INP            ; Read Input Dip-SW4
            ANDA    #$F0
            CMPA    INPUT_BUF
            BEQ     READ_INP
            STAA    INPUT_BUF           ; Update Buffer
            ;
SHOW_INP    LDAA    #$4C                ; Cursor Display
            JSR     GOTO_LCD
            LDAB    INPUT_BUF
            COMB                        ; Invert Logic
            LSRB                        ; Shift D4 to D0
            LSRB
            LSRB
            LSRB
            CLRA
            LSRB                        ; Shift Bit Dip-1 to Carry
            ADCA    #$30                ; Convert to ASCII
            JSR     WR_LCD
            CLRA
            LSRB                        ; Shift Bit Dip-2 to Carry
            ADCA    #$30                ; Convert to ASCII
            JSR     WR_LCD
            CLRA
            LSRB                        ; Shift Bit Dip-3 to Carry
            ADCA    #$30                ; Convert to ASCII
            JSR     WR_LCD
            CLRA
            LSRB                        ; Shift Bit Dip-4 to Carry
            ADCA    #$30                ; Convert to ASCII
            JSR     WR_LCD
            BRA     READ_INP
            
;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PB0   */;
;/* -> E    = PB2   */;
;/* -> D4-7 = PB4-7 */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0C                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor off,not blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
            LDAA    #$01                ; Clear Display
            JSR     WR_INS              ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      PSHB
            LDAB    #20                 ; 100uS counter
BUSY        DECB
            BNE     BUSY
            PULB
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0632              ; 5 mSec counter
INS_DLY     DEX
            BNE     INS_DLY
            PULX
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

;/********************************/;
;/* Print ASCII data to LCD Port */;
;/* Usage    : JSR  PRINT_LCD    */;
;/*          : FCC  'xxxxxxxx'   */;
;/*          : FCB  $00          */;
;/* Note     : last byte = $00   */;
;/********************************/;
;
PRINT_LCD   PULX                        ; Pop Stack
            PSHA
            PSHB
PRINT1      LDAA    0,X                 ; Get Data For Print
            CMPA    #$00                ; Check Last Byte
            BNE     PRINT2              ; Jump if not Last Byte
            BRA     PRINT3
PRINT2      JSR     WR_LCD              ; Write Data to LCD
            INX
            BRA     PRINT1
PRINT3      INX
            PULB
            PULA
            PSHX                        ; Push Stack
            RTS

            ORG     $FFFE               ; Reset Vector
            FDB     START

            END


