;/********************************/;
;/* Demo ADC LTC1298 For ET-EXP4 */;
;/* Files Name  : LAB7-EX1.ASM   */;
;/* CPU Control : 68HC811E2      */;
;/* Assembler   : AS11NEW.EXE    */;
;/* Display ADC Result on LCD16X2*/;
;/********************************/;
;
;/* Interface SPI With LTC1298
;/* CPU MISO (PD2) -- ADC DOUT (Pin 6)
;/* CPU MOSI (PD3) -- ADC DIN  (Pin 5)
;/* CPU SCK  (PD4) -- ADC CLK  (Pin 7)
;/* CPU SS#  (PD1) -- ADC CS   (Pin 1)
;
PORTD       EQU     $1008               ; PortD Data Register
DDRD        EQU     $1009               ; PortD Data Direction
SPCR        EQU     $1028               ; SPI Control Register
SPSR        EQU     $1029               ; SPI Status Register
SPDR        EQU     $102A               ; SPI Data Register
CS_EEP      EQU     #%00000001          ; CS EEP = PD0
CS_ADC      EQU     #%00000010          ; CS ADC = PD1
;
;/* Port For Control LCD (Port-B)
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)

            ORG     $00
ADC_BUFF    RMB     2                   ; ADC Result

            ORG     $F800               ; Start Monitor of 68H1811E2
START       LDS     #$FF                ; Stack on Last Internal RAM
            LDAA    #%00111011          ; PD2=In:PD0,1,3,4,5=Out
            STAA    DDRD
            LDX     #$1000              ; X = Base Register
            BCLR    $08,X CS_EEP        ; PD0 = 0 = Stop EEP
            BSET    $08,X CS_ADC        ; PD1 = 1 = Stop ADC
            LDAA    #%01010000          ; SPI=Master,SCK Normal=0
            STAA    SPCR                ; Clock = E/32,Active Rising
            ;
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            LDAA    #$00                ; 1st Location Line1
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'DEMO ADC LTC1298'  ; Message on Line-1
            FCB     $00
            LDAA    #$40                ; 1st Location Line2
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'ADC Result :    '  ; Message on Line-2
            FCB     $00
            ;
            ;/* 3 Byte SPI For Control Word LTC1298
            ;/* 1st Byte = [X]  [X]  [X]   [X][X][X][X][Start]
            ;/* 2nd Byte = [SGL][ODD][MSBF][X][X][X][X][X]
            ;/* 3rd Byte = [X]  [X]  [X]   [X][X][X][X][X]
            ;/* XXXXXXX1-101XXXXX-XXXXXXXX = CH0:IN,GND
            ;/* XXXXXXX1-111XXXXX-XXXXXXXX = CH1:IN,GND
            ;/* XXXXXXX1-001XXXXX-XXXXXXXX = CH0:IN,CH1=GND
            ;/* XXXXXXX1-011XXXXX-XXXXXXXX = CH1:IN,CH0=GND
            ;/* Replace X = 0
            ;
LOOP_CH0    BCLR    $08,X CS_ADC        ; CS = 0 = Active
            LDAA    #%00000001          ; Start Bit
            STAA    SPDR                ; Send SPI Byte1
WAIT_OK1    LDAA    SPSR                ; Read Status
            BPL     WAIT_OK1
            ;
            LDAA    #%10100000          ; SGL=Single:Ch0,MSBF = MSB First
            STAA    SPDR                ; Send SPI Byte2
WAIT_OK2    LDAA    SPSR                ; Read Status
            BPL     WAIT_OK2
            ;
            LDAA    SPDR                ; Read Result ADC High-Byte
            STAA    ADC_BUFF+0
            ;
            LDAA    #%00000000          ; Dummy Byte
            STAA    SPDR                ; Send SPI Byte3
WAIT_OK3    LDAA    SPSR
            BPL     WAIT_OK3
            ;
            BSET    $08,X CS_ADC        ; CS = 1 = Stop
            LDAA    SPDR                ; Read Result ADC Low-Byte
            STAA    ADC_BUFF+1
            ;
            ;/* Display Result on LCD 16x2
            LDAA    #$4D                ; Set Position Cursor Display
            JSR     GOTO_LCD
            LDAA    ADC_BUFF+0          ; High Byte Result
            ANDA    #$0F
            JSR     DSP_ASCII
            LDAA    ADC_BUFF+1          ; Low Byte Result
            LSRA                        ; Shift High to Low
            LSRA
            LSRA
            LSRA
            JSR     DSP_ASCII
            LDAA    ADC_BUFF+1
            ANDA    #$0F
            JSR     DSP_ASCII
            JMP     LOOP_CH0

;/************************/;
;/* Display ASCII to LCD */;
;/* Input : ACCA (HEX)   */;
;/************************/;
;
DSP_ASCII   CMPA    #$09                ; Convert to ASCII
            BHI     DSP_ASC1
            ADDA    #$30                ; Convert 0-9 to ASCII
            JSR     WR_LCD
            BRA     DSP_ASC2
            ;
DSP_ASC1    SBCA    #$09                ; Convert A-F to ASCII
            ADDA    #$40
            JSR     WR_LCD
DSP_ASC2    RTS

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0C                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor off,not blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
            LDAA    #$01                ; Clear display
            JSR     WR_INS              ; clear display,set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      JSR     BUSY                ; Delay 5mS
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/***********************/;
;/* Delay time for Busy */;
;/*    Wait LCD Ready   */;
;/***********************/;
;
BUSY        PSHX
            LDX     #$0632              ; 5 mSec counter
BUSY_LCD1   DEX
            BNE     BUSY_LCD1
            PULX
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

;/********************************/;
;/* Print ASCII data to LCD Port */;
;/* Usage    : JSR  PRINT_LCD    */;
;/*          : FCC  'xxxxxxxx'   */;
;/*          : FCB  $00          */;
;/* Note     : last byte = $00   */;
;/********************************/;
;
PRINT_LCD   PULY                        ; Pop Stack
            PSHA
            PSHB
PRINT1      LDAA    0,Y                 ; Get Data For Print
            CMPA    #$00                ; Check Last Byte
            BNE     PRINT2              ; Jump if not Last Byte
            BRA     PRINT3
PRINT2      JSR     WR_LCD              ; Write Data to LCD
            INY
            BRA     PRINT1
PRINT3      INY
            PULB
            PULA
            PSHY                        ; Push Stack
            RTS

            ORG     $FFFE               ; Reset Vector
            FDB     START

            END

